@@ -18,7 +18,12 @@ import dataclasses
 import datetime
 import typing
 from typing import Container, Protocol, Sequence
-from orbax.checkpoint._src.metadata import checkpoint_info
+from orbax.checkpoint import options as options_lib
+from orbax.checkpoint._src.checkpoint_managers import policy_checkpoint_info
+from orbax.checkpoint._src.multihost import multihost
+
+
+PolicyCheckpointInfo = policy_checkpoint_info.PolicyCheckpointInfo
 
 
 @dataclasses.dataclass(kw_only=True)
@@ -27,6 +32,7 @@ class DecisionContext:
 
   is_saving_in_progress: bool
   reached_preemption: bool
+  multiprocessing_options: options_lib.MultiprocessingOptions
 
 
 @typing.runtime_checkable
@@ -39,8 +45,8 @@ class SaveDecisionPolicy(Protocol):
 
   def should_save(
       self,
-      step: checkpoint_info.CheckpointInfo,
-      previous_steps: Sequence[checkpoint_info.CheckpointInfo],
+      step: PolicyCheckpointInfo,
+      previous_steps: Sequence[PolicyCheckpointInfo],
       *,
       context: DecisionContext
   ) -> bool:
@@ -55,8 +61,8 @@ class FixedIntervalPolicy(SaveDecisionPolicy):
 
   def should_save(
       self,
-      step: checkpoint_info.CheckpointInfo,
-      previous_steps: Sequence[checkpoint_info.CheckpointInfo],
+      step: PolicyCheckpointInfo,
+      previous_steps: Sequence[PolicyCheckpointInfo],
       *,
       context: DecisionContext
   ) -> bool:
@@ -73,8 +79,8 @@ class SpecificStepsPolicy(SaveDecisionPolicy):
 
   def should_save(
       self,
-      step: checkpoint_info.CheckpointInfo,
-      previous_steps: Sequence[checkpoint_info.CheckpointInfo],
+      step: PolicyCheckpointInfo,
+      previous_steps: Sequence[PolicyCheckpointInfo],
       *,
       context: DecisionContext
   ) -> bool:
@@ -91,8 +97,8 @@ class ContinuousCheckpointingPolicy(SaveDecisionPolicy):
 
   def should_save(
       self,
-      step: checkpoint_info.CheckpointInfo,
-      previous_steps: Sequence[checkpoint_info.CheckpointInfo],
+      step: PolicyCheckpointInfo,
+      previous_steps: Sequence[PolicyCheckpointInfo],
       *,
       context: DecisionContext
   ) -> bool:
@@ -100,9 +106,19 @@ class ContinuousCheckpointingPolicy(SaveDecisionPolicy):
       return False
     if not previous_steps or self.minimum_interval_secs is None:
       return True
-    return step.time - previous_steps[-1].time >= datetime.timedelta(
-        seconds=self.minimum_interval_secs
+    save_result = False
+    is_primary_host = multihost.is_primary_host(
+        context.multiprocessing_options.primary_host
     )
+    # Make time based decision only on primary host and broadcast to all hosts.
+    if is_primary_host:
+      save_result = step.time - previous_steps[-1].time >= datetime.timedelta(
+          seconds=self.minimum_interval_secs
+      )
+    save_result = bool(
+        multihost.broadcast_one_to_all(save_result, is_source=is_primary_host)
+    )
+    return save_result
 
 
 class PreemptionCheckpointingPolicy(SaveDecisionPolicy):
@@ -110,8 +126,8 @@ class PreemptionCheckpointingPolicy(SaveDecisionPolicy):
 
   def should_save(
       self,
-      step: checkpoint_info.CheckpointInfo,
-      previous_steps: Sequence[checkpoint_info.CheckpointInfo],
+      step: PolicyCheckpointInfo,
+      previous_steps: Sequence[PolicyCheckpointInfo],
       *,
       context: DecisionContext
   ) -> bool:
@@ -125,8 +141,8 @@ class InitialSavePolicy(SaveDecisionPolicy):
 
   def should_save(
       self,
-      step: checkpoint_info.CheckpointInfo,
-      previous_steps: Sequence[checkpoint_info.CheckpointInfo],
+      step: PolicyCheckpointInfo,
+      previous_steps: Sequence[PolicyCheckpointInfo],
       *,
       context: DecisionContext
   ) -> bool:
@@ -147,8 +163,8 @@ class AnySavePolicy(SaveDecisionPolicy):
 
   def should_save(
       self,
-      step: checkpoint_info.CheckpointInfo,
-      previous_steps: Sequence[checkpoint_info.CheckpointInfo],
+      step: PolicyCheckpointInfo,
+      previous_steps: Sequence[PolicyCheckpointInfo],
       *,
       context: DecisionContext
   ) -> bool:
