@@ -78,6 +78,7 @@ RestoreTransform = transform_utils.RestoreTransform
 PyTreeCheckpointHandler = test_utils.PyTreeCheckpointHandler
 _SHARDING = '_sharding'
 PYTREE_METADATA_FILE = pytree_checkpoint_handler.PYTREE_METADATA_FILE
+PLACEHOLDER = base_pytree_checkpoint_handler.PLACEHOLDER
 
 
 ARRAY_METADATA_STORE = array_metadata_store_lib.Store()
@@ -2378,3 +2379,195 @@ class PyTreeCheckpointHandlerTestBase:
             self.directory, args=PyTreeRestoreArgs(pytree, restore_args)
         )
         test_utils.assert_tree_equal(self, pytree, restored)
+
+    def test_pinned_host_loading(self):
+      if multihost.is_pathways_backend():
+        # TODO(b/404915487): Reenable when possible.
+        self.skipTest('Disabled due to b/404915487.')
+      pytree = dict(arr=np.ones((1024, 512)))
+      self.handler.save(self.directory, args=PyTreeSaveArgs(pytree))
+
+      mesh = jax.sharding.Mesh(
+          np.asarray(jax.devices()).reshape((1, len(jax.devices()))), ('x', 'y')
+      )
+      sharding = jax.sharding.NamedSharding(
+          mesh, jax.sharding.PartitionSpec('x', 'y')
+      ).with_memory_kind('pinned_host')
+
+      restore_args = dict(arr=ArrayRestoreArgs(sharding=sharding))
+      restored = self.handler.restore(
+          self.directory, args=PyTreeRestoreArgs(restore_args=restore_args)
+      )
+      expected = dict(arr=jax.device_put(np.ones((1024, 512)), sharding))
+      self.validate_restore(expected, restored)
+
+    @parameterized.product(
+        use_ocdbt=(True, False),
+        reference_item=(
+            {
+                'a': 0,
+                'b': 0,
+                'c': {
+                    'e': 0,
+                },
+            },
+            {
+                'a': 0,
+                'c': {
+                    'a': 0,
+                    'e': 0,
+                },
+            },
+            {
+                'a': 0,
+                'b': 0,
+            },
+        ),
+    )
+    def test_restore_item_has_missing_leaves(
+        self, use_ocdbt: bool, reference_item: dict[str, Any]
+    ):
+      with self.ocdbt_checkpoint_handler(
+          use_ocdbt=use_ocdbt,
+          array_metadata_store=array_metadata_store_lib.Store(),
+      ) as handler:
+        handler.save(self.directory, args=PyTreeSaveArgs(self.pytree))
+
+        with self.assertRaisesRegex(
+            ValueError, 'User-provided restore item and on-disk value'
+        ):
+          handler.restore(
+              self.directory,
+              item=reference_item,
+          )
+
+    def test_partial_restore_with_placeholder_simple(self):
+      original_item = {
+          'a': np.arange(8),
+          'b': np.arange(8),
+          'c': {
+              'a': np.arange(8),
+              'e': np.arange(8),
+          },
+      }
+      reference_item = {
+          'a': 0,
+          'b': PLACEHOLDER,
+          'c': {
+              'a': 0,
+              'e': PLACEHOLDER,
+          },
+      }
+      expected = {
+          'a': original_item['a'],
+          'b': PLACEHOLDER,
+          'c': {
+              'a': original_item['c']['a'],
+              'e': PLACEHOLDER,
+          },
+      }
+
+      simple_dir = epath.Path(
+          self.create_tempdir(name='simple_placeholder_dir').full_path
+      )
+
+      handler = PyTreeCheckpointHandler()
+      handler.save(simple_dir, args=PyTreeSaveArgs(original_item))
+      restored = handler.restore(
+          simple_dir,
+          args=PyTreeRestoreArgs(
+              item=reference_item,
+          ),
+      )
+      test_utils.assert_tree_equal(self, expected, restored)
+
+    @parameterized.product(use_ocdbt=(True, False))
+    def test_partial_restore_with_placeholder(self, use_ocdbt: bool):
+      """Test saving and restoring placeholder."""
+      with self.ocdbt_checkpoint_handler(
+          use_ocdbt=use_ocdbt,
+          array_metadata_store=array_metadata_store_lib.Store(),
+      ) as save_handler:
+        save_handler.save(self.directory, args=PyTreeSaveArgs(self.pytree))
+
+      restore_args = jax.tree.map(
+          lambda arr: ArrayRestoreArgs(sharding=arr.sharding), self.pytree
+      )
+
+      with self.subTest('success'):
+        reference_item = {
+            'a': 0,
+            'b': PLACEHOLDER,
+            'c': {
+                'a': 0,
+                'e': PLACEHOLDER,
+            },
+        }
+        expected = {
+            'a': self.pytree['a'],
+            'b': PLACEHOLDER,
+            'c': {
+                'a': self.pytree['c']['a'],
+                'e': PLACEHOLDER,
+            },
+        }
+
+        with self.ocdbt_checkpoint_handler(
+            use_ocdbt=use_ocdbt,
+            array_metadata_store=array_metadata_store_lib.Store(),
+        ) as restore_handler:
+          restored = restore_handler.restore(
+              self.directory,
+              args=PyTreeRestoreArgs(
+                  item=reference_item,
+                  restore_args=restore_args,
+              ),
+          )
+          test_utils.assert_tree_equal(self, expected, restored)
+
+      with self.subTest('missing_leaf'):
+        reference_item = {
+            'a': 0,
+            'b': PLACEHOLDER,
+            'c': {
+                # 'a': 0,
+                'e': PLACEHOLDER,
+            },
+        }
+
+        with self.ocdbt_checkpoint_handler(
+            use_ocdbt=use_ocdbt,
+            array_metadata_store=array_metadata_store_lib.Store(),
+        ) as restore_handler:
+          with self.assertRaisesRegex(
+              ValueError, 'User-provided restore item and on-disk value'
+          ):
+            restore_handler.restore(
+                self.directory,
+                args=PyTreeRestoreArgs(
+                    item=reference_item,
+                    restore_args=restore_args,
+                ),
+            )
+
+      with self.subTest('non_leaf_placeholder'):
+        reference_item = {
+            'a': 0,
+            'b': 0,
+            'c': PLACEHOLDER,
+        }
+
+        with self.ocdbt_checkpoint_handler(
+            use_ocdbt=use_ocdbt,
+            array_metadata_store=array_metadata_store_lib.Store(),
+        ) as restore_handler:
+          with self.assertRaisesRegex(
+              ValueError, 'User-provided restore item and on-disk value'
+          ):
+            restore_handler.restore(
+                self.directory,
+                args=PyTreeRestoreArgs(
+                    item=reference_item,
+                    restore_args=restore_args,
+                ),
+            )
