@@ -26,9 +26,20 @@ from orbax.checkpoint._src.path import atomicity_types
 from orbax.checkpoint._src.path import step as step_lib
 
 
-def get_default_temporary_path_class(
+def get_item_default_temporary_path_class(
     final_path: epath.Path,
 ) -> Type[atomicity_types.TemporaryPath]:
+  """Returns the default temporary path class for a given sub-item path."""
+  if step_lib.is_gcs_path(final_path):
+    return atomicity.CommitFileTemporaryPath
+  else:
+    return atomicity.AtomicRenameTemporaryPath
+
+
+def get_default_temporary_path_class(
+    final_path: epath.Path,
+) -> Type[atomicity_types.TemporaryPath]:
+  """Returns the default temporary path class for a given checkpoint path."""
   if step_lib.is_gcs_path(final_path):
     return atomicity.CommitFileTemporaryPath
   else:
