@@ -14,6 +14,7 @@
 
 """TensorStore serialization helper functions."""
 
+import copy
 import math
 import os
 import re
@@ -71,7 +72,12 @@ _REMOTE_DRIVER_VALIDATIONS = [
 
 
 
-def get_ts_context(*, use_ocdbt: bool = True) -> ts.Context:
+def get_ts_context(
+    *,
+    use_ocdbt: bool = True,
+    file_io_concurrency_limit: int | None = None,
+    data_copy_concurrency_limit: int | None = None,
+) -> ts.Context:
   """Creates a TensorStore context object.
 
   For use with Orbax serialization APIs, or when directly opening a
@@ -80,14 +86,26 @@ def get_ts_context(*, use_ocdbt: bool = True) -> ts.Context:
   Args:
     use_ocdbt: Whether to use OCDBT driver. Adds options specific to OCDBT if
       True.
+    file_io_concurrency_limit: Optionally overrides the thread pool size for
+      file I/O.
+    data_copy_concurrency_limit: Optionally overrides the thread pool size for
+      compressing and copying data.
 
   Returns:
     A TensorStore context object.
   """
-  if use_ocdbt:
-    return ts.Context(_DEFAULT_OCDBT_TS_CONTEXT)
-  else:
-    return ts.Context(_BASE_TS_CONTEXT)
+  context = copy.deepcopy(
+      _DEFAULT_OCDBT_TS_CONTEXT if use_ocdbt else _BASE_TS_CONTEXT
+  )
+  if file_io_concurrency_limit is not None:
+    context.setdefault('file_io_concurrency', {})[
+        'limit'
+    ] = file_io_concurrency_limit
+  if data_copy_concurrency_limit is not None:
+    context.setdefault('data_copy_concurrency', {})[
+        'limit'
+    ] = data_copy_concurrency_limit
+  return ts.Context(context)
 
 
 ### Building KvStore specs.
@@ -222,6 +240,7 @@ def build_zarr_shard_and_chunk_metadata(
     *,
     global_shape: Shape,
     shard_shape: Shape,
+    use_zarr2_compression: bool = True,
     use_zarr3: bool,
     chunk_shape: Shape,
 ) -> JsonSpec:
@@ -231,7 +250,10 @@ def build_zarr_shard_and_chunk_metadata(
   if not use_zarr3:
     # Zarr v2.
     metadata['chunks'] = chunk_shape
-    metadata['compressor'] = {'id': 'zstd'}
+    if use_zarr2_compression:
+      metadata['compressor'] = {'id': 'zstd'}
+    else:
+      metadata['compressor'] = None
   else:
     # Zarr v3.
     metadata['chunk_grid'] = {
@@ -333,6 +355,7 @@ class ArrayWriteSpec:
       target_dtype: DType | None = None,
       chunk_byte_size: int | None = None,
       shard_axes: tuple[int, ...] = (),
+      use_zarr2_compression: bool = True,
       use_zarr3: bool = False,
       use_ocdbt: bool,
       ocdbt_target_data_file_size: int | None = None,
@@ -394,6 +417,7 @@ class ArrayWriteSpec:
     tspec['metadata'] = build_zarr_shard_and_chunk_metadata(
         global_shape=global_shape,
         shard_shape=write_shape,
+        use_zarr2_compression=use_zarr2_compression,
         use_zarr3=use_zarr3,
         chunk_shape=chunk_shape,
     )
