@@ -60,6 +60,7 @@ TypeHandler: TypeAlias = types.TypeHandler
 TypeHandlerRegistry: TypeAlias = types.TypeHandlerRegistry
 Pytree = Any
 PLACEHOLDER = ...
+PLACEHOLDER_TYPESTR = 'placeholder'
 
 Layout = layout.Layout
 Shape = arrays_types.Shape
@@ -288,7 +289,7 @@ async def _validate_params(
 
   # TODO: b/362328389 - Add support for zarr3.
   if use_zarr3:
-    logging.warning(
+    logging.info(
         'Param validation support for Zarr3 will be added later (b/362328389).'
     )
     return
@@ -298,7 +299,7 @@ async def _validate_params(
     # TODO: b/361090820 - Raise error once we confirm that Bennu writing empty
     # states is a bug.
     # e.g. //learning/deepmind/jax/roc/formats/roc_orbax:roc_orbax_test
-    logging.warning(
+    logging.info(
         'Skipping param validation: No params found in TensorStore'
         ' KvStore: %s.',
         ts_kv_store,
@@ -1194,7 +1195,7 @@ class ArrayHandler(types.TypeHandler):
         if meta.ext_metadata is None or not isinstance(meta.ext_metadata, dict):
           continue
 
-        if impl := meta.ext_metadata.get(array_metadata_lib.RANDOM_KEY_IMPL):
+        if impl := meta.ext_metadata.get(array_metadata_lib.RANDOM_KEY_IMPL):  # pytype: disable=attribute-error
           deserialized_arrays[i] = jax.random.wrap_key_data(v, impl=impl)
           logging.vlog(
               1,
@@ -1248,12 +1249,11 @@ class ArrayHandler(types.TypeHandler):
           sharding = arg.sharding
       elif sharding_file_exists:
         warnings.warn(
-            "Couldn't find sharding info under RestoreArgs. Populating sharding"
+            'Sharding info not provided when restoring. Populating sharding'
             ' info from sharding file. Please note restoration time will be'
-            ' slightly increased due to reading from file instead of directly'
-            ' from RestoreArgs. Note also that this option is unsafe when'
-            ' restoring on a different topology than the checkpoint was saved'
-            ' with.'
+            ' slightly increased due to reading from file. Note also that this'
+            ' option is unsafe when restoring on a different topology than the'
+            ' checkpoint was saved with.'
         )
         assert info.parent_dir is not None
         if info.name:
@@ -1374,23 +1374,23 @@ class SingleReplicaArrayHandler(ArrayHandler):
   Consider, for example, the following sharding on v4-128 which has 16 hosts and
   64 devices::
 
-      shape = (32, 2)
-      mesh = jax.sharding.Mesh(jax.devices().reshape(shape), ('x', 'y'))
-      pspec = jax.sharding.PartitionSpec(None, 'y')
-      sharding=jax.sharding.NamedSharding(mesh, pspec)
+    shape = (32, 2)
+    mesh = jax.sharding.Mesh(jax.devices().reshape(shape), ('x', 'y'))
+    pspec = jax.sharding.PartitionSpec(None, 'y')
+    sharding=jax.sharding.NamedSharding(mesh, pspec)
 
   This sharding will not work since the primary replica has only two devices,
   and hence there is a host which has 2 devices in the primary replica, and 2
   devices outside of primary replica. However, changing shape, for example, to
   (4, 16) will result in a valid sharding.
 
-  This TypeHandler can be registered by running
-  ```
-  ocp.type_handlers.register_type_handler(
-      jax.Array,
-      type_handlers.SingleReplicaArrayHandler(),
-      override=True)
-  ```
+  This TypeHandler can be registered by running::
+
+    ocp.type_handlers.register_type_handler(
+        jax.Array,
+        type_handlers.SingleReplicaArrayHandler(),
+        override=True)
+
   Example usage can be found in MaxText (TO BE MERGED).
   https://github.com/google/maxtext/blob/main/MaxText/checkpointing.py
   """
@@ -1694,11 +1694,15 @@ class StringHandler(types.TypeHandler):
     return list(zip(write_sizes, read_sizes))
 
 
+def is_placeholder(value: Any) -> bool:
+  return value is PLACEHOLDER
+
+
 class PlaceholderHandler(types.TypeHandler):
   """TypeHandler for placeholders."""
 
   def typestr(self) -> str:
-    return 'placeholder'
+    return PLACEHOLDER_TYPESTR
 
   async def metadata(self, infos: Sequence[types.ParamInfo]) -> Sequence[Any]:
     raise NotImplementedError('Placeholders do not have metadata.')
