@@ -16,6 +16,8 @@
 
 # pylint: disable=protected-access, missing-function-docstring
 
+from __future__ import annotations
+
 import asyncio
 import contextlib
 import copy
@@ -39,7 +41,6 @@ import optax
 from orbax.checkpoint import test_utils
 from orbax.checkpoint import utils
 from orbax.checkpoint._src.arrays import abstract_arrays
-from orbax.checkpoint._src.handlers import proto_checkpoint_handler
 from orbax.checkpoint._src.handlers import pytree_checkpoint_handler
 from orbax.checkpoint._src.metadata import array_metadata
 from orbax.checkpoint._src.metadata import array_metadata_store as array_metadata_store_lib
@@ -47,7 +48,6 @@ from orbax.checkpoint._src.metadata import empty_values
 from orbax.checkpoint._src.metadata import sharding as sharding_metadata
 from orbax.checkpoint._src.metadata import tree as tree_metadata
 from orbax.checkpoint._src.metadata import value as value_metadata
-from orbax.checkpoint._src.multihost import multihost
 from orbax.checkpoint._src.serialization import replica_slices
 from orbax.checkpoint._src.serialization import serialization
 from orbax.checkpoint._src.serialization import tensorstore_utils as ts_utils
@@ -56,7 +56,11 @@ from orbax.checkpoint._src.tree import utils as tree_utils
 from orbax.checkpoint.experimental.v1._src.context import context as context_lib
 from orbax.checkpoint.experimental.v1._src.context import options as options_lib
 from orbax.checkpoint.experimental.v1._src.handlers import pytree_handler
+from orbax.checkpoint.experimental.v1._src.path import types as path_types
+from orbax.checkpoint.experimental.v1._src.serialization import registration as serialization_registration
+from orbax.checkpoint.experimental.v1._src.synchronization import multihost
 from orbax.checkpoint.experimental.v1._src.testing import array_utils as array_test_utils
+from orbax.checkpoint.experimental.v1._src.testing import path_utils as path_test_utils
 from orbax.checkpoint.experimental.v1._src.tree import types as tree_types
 
 
@@ -66,6 +70,7 @@ ParamInfo = pytree_checkpoint_handler.ParamInfo
 _SHARDING = '_sharding'
 PYTREE_METADATA_FILE = pytree_checkpoint_handler.PYTREE_METADATA_FILE
 ARRAY_METADATA_STORE = array_metadata_store_lib.Store()
+PLACEHOLDER = type_handlers.PLACEHOLDER
 
 create_sharded_array = array_test_utils.create_sharded_array
 create_numpy_pytree = array_test_utils.create_numpy_pytree
@@ -73,6 +78,11 @@ create_sharded_pytree = array_test_utils.create_sharded_pytree
 as_abstract_type = array_test_utils.as_abstract_type
 
 
+PathAwaitingCreation = path_types.PathAwaitingCreation
+PathLike = path_types.PathLike
+Path = path_types.Path
+
+
 async def _run_awaitable(awaitable: Awaitable[Any]) -> Any:
   return await awaitable
 
@@ -83,22 +93,25 @@ class PyTreeHandler:
   def __init__(self, **kwargs):
     self._handler = pytree_handler.PyTreeHandler(**kwargs)
 
-  def save(self, *args, **kwargs):
-    awaitable = asyncio.run(self._handler.save(*args, **kwargs))
+  def save(self, path: Path, checkpointable: PyTree):
+    awaitable = self.save_async(path, checkpointable)
     return asyncio.run(_run_awaitable(awaitable))
 
-  def save_async(self, *args, **kwargs):
-    return asyncio.run(self._handler.save(*args, **kwargs))
+  def save_async(self, path: Path, checkpointable: PyTree):
+    path = path_test_utils.PathAwaitingCreationWrapper(path)
+    return asyncio.run(self._handler.save(path, checkpointable))
 
-  def load(self, *args, **kwargs):
-    awaitable = asyncio.run(self._handler.load(*args, **kwargs))
+  def load(self, path: Path, abstract_checkpointable: PyTree | None = None):
+    awaitable = self.load_async(path, abstract_checkpointable)
     return asyncio.run(_run_awaitable(awaitable))
 
-  def load_async(self, *args, **kwargs):
-    return asyncio.run(self._handler.load(*args, **kwargs))
+  def load_async(
+      self, path: Path, abstract_checkpointable: PyTree | None = None
+  ):
+    return asyncio.run(self._handler.load(path, abstract_checkpointable))
 
-  def metadata(self, *args, **kwargs):
-    return asyncio.run(self._handler.metadata(*args, **kwargs))
+  def metadata(self, path: Path):
+    return asyncio.run(self._handler.metadata(path))
 
 
 def create_mixed_format_pytree(
@@ -162,6 +175,16 @@ def init_flax_model(model):
   return jax.tree.map(np.asarray, state)
 
 
+def get_d_files(path: Path) -> list[Path]:
+  files = []
+  for idx in range(multihost.process_count()):
+    d_path = path / f'ocdbt.process_{idx}' / 'd'
+    if not d_path.exists():
+      continue
+    files.extend(list(d_path.iterdir()))
+  return files
+
+
 @contextlib.contextmanager
 def handler_with_options(
     *,
@@ -336,9 +359,7 @@ class PyTreeHandlerTestBase:
           expected_reference_metadata_tree,
           is_leaf=tree_utils.is_empty_or_leaf,
       )
-      test_utils.assert_tree_equal(
-          self, expected_metadata, actual_metadata.tree
-      )
+      test_utils.assert_tree_equal(self, expected_metadata, actual_metadata)
 
     def test_get_param_names(self):
       param_names = pytree_checkpoint_handler.get_param_names(self.pytree)
@@ -735,31 +756,59 @@ class PyTreeHandlerTestBase:
         )
 
     def test_save_async(self):
-      save_done = threading.Event()
-      original_finalize = pytree_handler.PyTreeHandler._finalize
+      # The pytree must be larger so that saving doesn't complete too quickly.
+      mesh = jax.sharding.Mesh(jax.devices(), 'x')
+      np.random.seed(42)
+      pytree = {
+          'a': array_test_utils.create_sharded_array(
+              np.arange(2**20),
+              sharding=jax.sharding.NamedSharding(
+                  mesh, jax.sharding.PartitionSpec('x')
+              ),
+          ),
+          'b': array_test_utils.create_sharded_array(
+              np.random.uniform(size=2**15),
+              sharding=jax.sharding.NamedSharding(
+                  mesh, jax.sharding.PartitionSpec(None)
+              ),
+          ),
+      }
+      abstract_pytree = jax.tree.map(array_test_utils.as_abstract_type, pytree)
 
-      def mock_finalize(*args, **kwargs):
-        save_done.wait()  # Wait for explicit signal before proceeding.
-        return original_finalize(*args, **kwargs)
+      start_serialize = threading.Event()
+      original_serialize = serialization.async_serialize_from_host
+
+      def mock_serialize(*args, **kwargs):
+        start_serialize.wait()  # Wait for explicit signal before proceeding.
+        return original_serialize(*args, **kwargs)
 
       def is_save_complete(directory):
         return (directory / 'manifest.ocdbt').exists()
 
-      with mock.patch.object(
-          pytree_handler.PyTreeHandler,
-          '_finalize',
-          new=mock_finalize,
-      ), handler_with_options() as checkpoint_handler:
-        awaitable = checkpoint_handler.save_async(self.directory, self.pytree)
+      # Serialization to disk does not start until receiving an explicit signal.
+      self.enter_context(
+          mock.patch.object(
+              serialization, 'async_serialize_from_host', new=mock_serialize
+          )
+      )
+
+      with handler_with_options() as checkpoint_handler:
+        awaitable = checkpoint_handler.save_async(self.directory, pytree)
+        initial_d_files = get_d_files(self.directory)
         self.assertFalse(is_save_complete(self.directory))
-        save_done.set()
+        start_serialize.set()
+
         asyncio.run(_run_awaitable(awaitable))
+        final_d_files = get_d_files(self.directory)
+        self.assertNotEmpty(final_d_files)
+        self.assertNotEqual(len(initial_d_files), len(final_d_files))
         self.assertTrue(is_save_complete(self.directory))
+
         restored = checkpoint_handler.load(
             self.directory,
-            self.abstract_pytree,
+            abstract_pytree,
         )
-        test_utils.assert_tree_equal(self, self.pytree, restored)
+        test_utils.assert_tree_equal(self, pytree, restored)
 
     def test_load_async(self):
       with handler_with_options() as checkpoint_handler:
@@ -1390,7 +1439,6 @@ class PyTreeHandlerTestBase:
       expected = {'a': 4}
       self.assertEqual(restored, expected)
 
-
     def test_empty_custom_node(self):
 
       class PyTreeDict(dict):
@@ -1600,7 +1648,7 @@ class PyTreeHandlerTestBase:
         self.assertFalse((self.directory / 'array_metadatas').exists())
         metadata = checkpoint_handler.metadata(self.directory)
         metadata_without_directory = jax.tree.map(
-            lambda m: dataclasses.replace(m, directory=None), metadata.tree
+            lambda m: dataclasses.replace(m, directory=None), metadata
         )
         self.assertEqual(
             expected_metadata_without_directory,
@@ -1637,7 +1685,7 @@ class PyTreeHandlerTestBase:
         }
         metadata = checkpoint_handler.metadata(self.directory)
         tree_with_write_shapes = jax.tree.map(
-            lambda m: {'write_shape': m.storage.write_shape}, metadata.tree
+            lambda m: {'write_shape': m.storage.write_shape}, metadata
         )
         self.assertDictEqual(
             expected_tree_with_write_shapes, tree_with_write_shapes
@@ -1677,7 +1725,7 @@ class PyTreeHandlerTestBase:
         }
         metadata = checkpoint_handler.metadata(self.directory)
         tree_with_write_shapes = jax.tree.map(
-            lambda m: {'write_shape': m.storage.write_shape}, metadata.tree
+            lambda m: {'write_shape': m.storage.write_shape}, metadata
         )
         self.assertDictEqual(
             expected_tree_with_write_shapes, tree_with_write_shapes
@@ -1879,3 +1927,146 @@ class PyTreeHandlerTestBase:
       ) as load_handler:
         restored = load_handler.load(self.directory)
         test_utils.assert_tree_equal(self, pytree, restored)
+
+    def test_pinned_host_loading(self):
+      if multihost.is_pathways_backend():
+        # TODO(b/404915487): Reenable when possible.
+        self.skipTest('Disabled due to b/404915487.')
+      pytree = dict(arr=np.ones((1024, 512)))
+      self.handler.save(self.directory, pytree)
+
+      mesh = jax.sharding.Mesh(
+          np.asarray(jax.devices()).reshape((1, len(jax.devices()))), ('x', 'y')
+      )
+      sharding = jax.sharding.NamedSharding(
+          mesh, jax.sharding.PartitionSpec('x', 'y')
+      ).with_memory_kind('pinned_host')
+
+      abstract_pytree = dict(
+          arr=jax.ShapeDtypeStruct(
+              pytree['arr'].shape, pytree['arr'].dtype, sharding=sharding
+          )
+      )
+      restored = self.handler.load(self.directory, abstract_pytree)
+      expected = dict(arr=jax.device_put(np.ones((1024, 512)), sharding))
+      test_utils.assert_tree_equal(self, expected, restored)
+
+    @parameterized.product(
+        use_ocdbt=(True, False),
+        reference_item=(
+            {
+                'a': 0,
+                'b': 0,
+                'c': {
+                    'e': 0,
+                },
+            },
+            {
+                'a': 0,
+                'c': {
+                    'a': 0,
+                    'e': 0,
+                },
+            },
+            {
+                'a': 0,
+                'b': 0,
+            },
+        ),
+    )
+    def test_restore_item_has_missing_leaves(
+        self, use_ocdbt: bool, reference_item: dict[str, Any]
+    ):
+      with handler_with_options(
+          use_ocdbt=use_ocdbt,
+          array_metadata_store=array_metadata_store_lib.Store(),
+      ) as handler:
+        handler.save(self.directory, self.pytree)
+
+        with self.assertRaisesRegex(
+            ValueError, 'User-provided restore item and on-disk value'
+        ):
+          handler.load(self.directory, reference_item)
+
+    def test_partial_restore_with_placeholder_simple(self):
+      original_item = {
+          'a': np.arange(8),
+          'b': np.arange(8),
+          'c': {
+              'a': np.arange(8),
+              'e': np.arange(8),
+          },
+      }
+      reference_item = jax.tree.map(as_abstract_type, original_item)
+      reference_item['b'] = PLACEHOLDER
+      reference_item['c']['e'] = PLACEHOLDER
+      expected = {
+          'a': original_item['a'],
+          'b': PLACEHOLDER,
+          'c': {
+              'a': original_item['c']['a'],
+              'e': PLACEHOLDER,
+          },
+      }
+
+      simple_dir = epath.Path(
+          self.create_tempdir(name='simple_placeholder_dir').full_path
+      )
+
+      handler = PyTreeHandler()
+      handler.save(simple_dir, original_item)
+      restored = handler.load(simple_dir, reference_item)
+      test_utils.assert_tree_equal(self, expected, restored)
+
+    @parameterized.product(use_ocdbt=(True, False))
+    def test_partial_restore_with_placeholder(self, use_ocdbt: bool):
+      """Test saving and restoring placeholder."""
+      with handler_with_options(
+          use_ocdbt=use_ocdbt,
+          array_metadata_store=array_metadata_store_lib.Store(),
+      ) as save_handler:
+        save_handler.save(self.directory, self.pytree)
+
+      with self.subTest('success'):
+        reference_item = self.abstract_pytree.copy()
+        reference_item['b'] = PLACEHOLDER
+        reference_item['c']['e'] = PLACEHOLDER
+
+        expected = self.pytree.copy()
+        expected['b'] = PLACEHOLDER
+        expected['c']['e'] = PLACEHOLDER
+
+        with handler_with_options(
+            use_ocdbt=use_ocdbt,
+            array_metadata_store=array_metadata_store_lib.Store(),
+        ) as restore_handler:
+          restored = restore_handler.load(self.directory, reference_item)
+          test_utils.assert_tree_equal(self, expected, restored)
+
+      with self.subTest('missing_leaf'):
+        reference_item = self.abstract_pytree.copy()
+        reference_item['b'] = PLACEHOLDER
+        reference_item['c']['e'] = PLACEHOLDER
+        del reference_item['c']['a']
+
+        with handler_with_options(
+            use_ocdbt=use_ocdbt,
+            array_metadata_store=array_metadata_store_lib.Store(),
+        ) as restore_handler:
+          with self.assertRaisesRegex(
+              ValueError, 'User-provided restore item and on-disk value'
+          ):
+            restore_handler.load(self.directory, reference_item)
+
+      with self.subTest('non_leaf_placeholder'):
+        reference_item = self.abstract_pytree.copy()
+        reference_item['c'] = PLACEHOLDER
+
+        with handler_with_options(
+            use_ocdbt=use_ocdbt,
+            array_metadata_store=array_metadata_store_lib.Store(),
+        ) as restore_handler:
+          with self.assertRaisesRegex(
+              ValueError, 'User-provided restore item and on-disk value'
+          ):
+            restore_handler.load(self.directory, reference_item)
