@@ -31,7 +31,7 @@ class AsyncOptions:
   timeout_secs: int = 600  # 10 minutes. Same as default in `AsyncCheckpointer`.
   barrier_sync_fn: Optional[multihost.BarrierSyncFn] = None
   post_finalization_callback: Optional[Callable[[], None]] = None
-  create_directories_asynchronously: bool = False
+  create_directories_asynchronously: bool = True
 
 
 @dataclasses.dataclass
@@ -57,6 +57,8 @@ class MultiprocessingOptions:
   barrier_sync_key_prefix: Optional[str] = None
 
 
+
+
 @dataclasses.dataclass(frozen=True)
 class FileOptions:
   """Options used to configure checkpoint directories and files.
